/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalVar.h"
#include "HvControlBinop.h"
#include "HvMath.h"
#include "HvControlCast.h"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvControlIf.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_nifWJMIG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VyadABLe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_oTiGeZax_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_q4DXHw2t_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ypv48ktb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IK2kav3M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_47fqLswy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_KnCjwItI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ThDPifIn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_9CbyWyoR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_lxG4vITA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_birP2wP1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vf2LF9bf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_IebJJz9d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_SFvlUC1l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_cKYWlT94_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_2Kqjd8n9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_poDHuXB6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_udnpAGgV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wXnEsDji_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_NN49bU6B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_2LOpi5qX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1Ax0qXRl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_jtuRcyR9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_uRiu1dZv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ltAMDdwF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ur8SqGfU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WgzaI8CX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZesgcPxj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oaVOFi20_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_CDyocVQP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_H8EIzTr0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_56B4zDqi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_fNipbqZz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_6Iggm2n1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_mvqQ1W1e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LZ6F4MGr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_OPsQDsDl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_FyQw8FC2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_X58ooILl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_v5xcdGS5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_idCgwBaT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_lKJPn9oB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BLgPmXY3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xb2a6PQ0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_w2eNYTHd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_s9AVsdLL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_CCXbSqyh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_RWlQVlUv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TlIs2xwG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_61EC08vP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Peby1fiV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_fGlrPJxZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CtHHgDNC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_UtucqgGc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4Fh8aki3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_FXyw5nZh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_aH02rMaE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_SAWPPY95_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_olgRsORH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_KSiB5Jdj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ShwjHQb0_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_1zMBns4z;
  SignalEnvelope sEnv_poDHuXB6;
  SignalEnvelope sEnv_udnpAGgV;
  SignalEnvelope sEnv_6Iggm2n1;
  SignalEnvelope sEnv_mvqQ1W1e;
  SignalLine sLine_NNMGFJBz;
  SignalLine sLine_4eNbfPr1;
  ControlBinop cBinop_nifWJMIG;
  ControlBinop cBinop_VyadABLe;
  ControlBinop cBinop_oTiGeZax;
  ControlBinop cBinop_q4DXHw2t;
  ControlBinop cBinop_47fqLswy;
  ControlBinop cBinop_ThDPifIn;
  ControlBinop cBinop_9CbyWyoR;
  ControlBinop cBinop_lxG4vITA;
  ControlVar cVar_birP2wP1;
  ControlVar cVar_IebJJz9d;
  SignalVarf sVarf_YY5DiygQ;
  ControlVar cVar_SFvlUC1l;
  ControlIf cIf_cKYWlT94;
  ControlIf cIf_2Kqjd8n9;
  ControlBinop cBinop_wXnEsDji;
  ControlBinop cBinop_2LOpi5qX;
  ControlBinop cBinop_1Ax0qXRl;
  ControlBinop cBinop_ltAMDdwF;
  ControlBinop cBinop_WgzaI8CX;
  SignalVarf sVarf_YtXLwxqr;
  ControlVar cVar_H8EIzTr0;
  ControlIf cIf_56B4zDqi;
  ControlIf cIf_fNipbqZz;
  ControlBinop cBinop_LZ6F4MGr;
  ControlBinop cBinop_FyQw8FC2;
  ControlBinop cBinop_X58ooILl;
  ControlBinop cBinop_lKJPn9oB;
  ControlBinop cBinop_xb2a6PQ0;
  ControlVar cVar_RWlQVlUv;
  ControlBinop cBinop_TlIs2xwG;
  SignalVarf sVarf_n0skG1iU;
  SignalVarf sVarf_2S1HYzAV;
  SignalVarf sVarf_TOHcVKMx;
  SignalVarf sVarf_1IPzlLdU;
  ControlBinop cBinop_CtHHgDNC;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
